@@ -2,10 +2,13 @@ import sys
 import uuid
 import argparse
 import logging
+import requests
 from colorama import Fore, Style
 from sumatra_client.config import CONFIG
 from sumatra_client.auth import CognitoAuth
 from sumatra_client.client import Client
+from sumatra_client.workspace import WorkspaceClient
+from sumatra_client.admin import AdminClient
 
 
 logger = logging.getLogger("sumatra.cli")
@@ -111,20 +114,20 @@ def config(args) -> None:
 
 
 def login(args) -> None:
-    instance = args.instance
     try:
-        curr_instance = CONFIG.instance
-    except:
-        curr_instance = None
-    while not instance:
-        prompt = "Sumatra Instance URL [%s]: " % (
-            curr_instance or "e.g.: yourco.sumatra.ai"
-        )
-        instance = input(prompt).strip()
-        if not instance:
-            instance = curr_instance
+        instance = args.instance or CONFIG.instance
+    except RuntimeError:
+        instance = "console.sumatra.ai"
+    if not args.instance:
+        prompt = f"Sumatra Instance URL [{instance}]: "
+        instance = input(prompt).strip() or instance
     CONFIG.instance = instance
-    CONFIG.update_from_stack()
+    try:
+        CONFIG.update_from_stack()
+    except requests.exceptions.ConnectionError:
+        print(f"Unable to connect to {instance}. Please check the URL and try again.")
+        sys.exit(1)
+
     CONFIG.save(update_default_instance=True)
 
     auth = CognitoAuth()
@@ -265,7 +268,10 @@ def table_delete(args):
 
 
 def table_show(args):
-    print(Client().get_tables().loc[args.table])
+    try:
+        print(Client().get_tables().loc[args.table])
+    except KeyError:
+        raise KeyError(f"Table '{args.table}' not found.")
 
 
 def table_schema(args):
@@ -290,7 +296,10 @@ def model_delete(args):
 
 
 def model_show(args):
-    print(Client().get_models().loc[args.model])
+    try:
+        print(Client().get_models().loc[args.model])
+    except KeyError:
+        raise KeyError(f"Model '{args.model}' not found.")
 
 
 def model_history(args):
@@ -307,6 +316,39 @@ def model_put(args):
     )
 
 
+def workspace_list(args):
+    workspaces = WorkspaceClient().get_workspaces().sort_index()
+    if args.show:
+        print(workspaces)
+    else:
+        print("\n".join(workspaces.index.to_list()))
+
+
+def workspace_delete(args):
+    WorkspaceClient().delete_workspace(args.workspace)
+    print(f"Successfully deleted workspace '{args.workspace}'")
+
+
+def workspace_create(args):
+    WorkspaceClient().create_workspace(args.workspace, args.nickname)
+    print(f"Successfully created workspace '{args.workspace}'")
+
+
+def workspace_show(args):
+    try:
+        print(WorkspaceClient().get_workspaces().loc[args.workspace])
+    except KeyError:
+        raise KeyError(f"Workspace '{args.workspace}' not found.")
+
+
+def workspace_select(args):
+    CONFIG.workspace = args.workspace
+    CONFIG.save(update_default_instance=False)
+    print(
+        f"Current workspace in config set to '{args.workspace}' for {CONFIG.instance}\n"
+    )
+
+
 def version(args):
     print(Client().version())
 
@@ -316,35 +358,28 @@ def tenant(args):
 
 
 def admin_list_tenants(args):
-    tenants = Client().list_tenants()
+    tenants = AdminClient().list_tenants()
     print("\n".join(tenants))
 
 
-def admin_current_tenant(args):
-    user = Client().get_user(args.user)
-    print(user["tenant"] + "\t" + user["role"])
-
-
-def admin_assign_tenant(args):
-    Client().set_tenant(args.user, args.tenant)
-    Client().set_role(args.user, args.role)
-
-    print(
-        f"Successfully assigned '{args.user}' to '{args.tenant}' with '{args.role}' role."
-    )
-
-
 def admin_list_users(args):
-    users = Client().list_users()
+    users = AdminClient().list_users()
     print("\n".join(users))
 
 
-def admin_show_keys(args):
-    print(Client().get_keys())
+def admin_upgrade_tenant(args):
+    AdminClient().upgrade_tenant(args.tenant)
+    print(f"Successfully upgraded tenant '{args.tenant}'")
 
 
-def admin_show_key_usage(args):
-    print(Client().get_key_usage())
+def admin_downgrade_tenant(args):
+    AdminClient().downgrade_tenant(args.tenant)
+    print(f"Successfully downgraded tenant '{args.tenant}'")
+
+
+def admin_set_quota(args):
+    AdminClient().set_quota(args.tenant, args.monthly_events)
+    print(f"Successfully set quota for tenant '{args.tenant}' to {args.monthly_events}")
 
 
 ADMIN_COMMANDS = [
@@ -353,30 +388,25 @@ ADMIN_COMMANDS = [
         "help": "list all tenants",
         "handler": admin_list_tenants,
     },
-    {
-        "name": "current-tenant",
-        "help": "show user's current tenant",
-        "handler": admin_current_tenant,
-    },
-    {
-        "name": "assign-tenant",
-        "help": "assign user to tenant",
-        "handler": admin_assign_tenant,
-    },
     {
         "name": "list-users",
         "help": "list all users",
         "handler": admin_list_users,
     },
     {
-        "name": "show-keys",
-        "help": "show api and sdk keys",
-        "handler": admin_show_keys,
+        "name": "upgrade",
+        "help": "upgrade a tenant to premium tier",
+        "handler": admin_upgrade_tenant,
     },
     {
-        "name": "show-key-usage",
-        "help": "show api and sdk key daily usage",
-        "handler": admin_show_key_usage,
+        "name": "downgrade",
+        "help": "downgrade a tenant to free tier",
+        "handler": admin_downgrade_tenant,
+    },
+    {
+        "name": "set-quota",
+        "help": "set tenant monthly event quota",
+        "handler": admin_set_quota,
     },
 ]
 
@@ -510,6 +540,34 @@ MODEL_COMMANDS = [
     },
 ]
 
+WORKSPACE_COMMANDS = [
+    {
+        "name": "list",
+        "help": "list all workspaces",
+        "handler": workspace_list,
+    },
+    {
+        "name": "delete",
+        "help": "delete workspace",
+        "handler": workspace_delete,
+    },
+    {
+        "name": "create",
+        "help": "create workspace",
+        "handler": workspace_create,
+    },
+    {
+        "name": "show",
+        "help": "display workspace metadata",
+        "handler": workspace_show,
+    },
+    {
+        "name": "select",
+        "help": "set current workspace in local config",
+        "handler": workspace_select,
+    },
+]
+
 COMMANDS = [
     {
         "name": "config",
@@ -566,6 +624,11 @@ COMMANDS = [
         "help": "run `sumatra timeline -h` for subcommands",
         "subcommands": TIMELINE_COMMANDS,
     },
+    {
+        "name": "workspace",
+        "help": "run `sumatra workspace -h` for subcommands",
+        "subcommands": WORKSPACE_COMMANDS,
+    },
     {
         "name": "admin",
         "help": "run `sumatra admin -h` for subcommands",
@@ -696,13 +759,23 @@ def main():
                             "file",
                             help="file with event data in JSON Lines format (may be gzipped)",
                         )
-                if cmd["name"] == "admin":
-                    if subcmd["name"] in ["current-tenant", "assign-tenant"]:
-                        p.add_argument("user", help="cognito login name of user")
-                    if subcmd["name"] == "assign-tenant":
-                        p.add_argument("tenant", help="tenant name")
+                if cmd["name"] == "workspace":
+                    if subcmd["name"] in ["delete", "show", "select", "create"]:
+                        p.add_argument("workspace", help="workspace name")
+                    if subcmd["name"] == "list":
                         p.add_argument(
-                            "role", help="role name (reader, writer, publisher)"
+                            "--show", action="store_true", help="display metadata"
+                        )
+                    if subcmd["name"] == "create":
+                        p.add_argument(
+                            "--nickname", help="specify optional workspace nickname"
+                        )
+                if cmd["name"] == "admin":
+                    if subcmd["name"] in ["upgrade", "downgrade", "set-quota"]:
+                        p.add_argument("tenant", help="tenant name")
+                    if subcmd["name"] == "set-quota":
+                        p.add_argument(
+                            "monthly_events", help="monthly event quota", type=int
                         )
 
     args = parser.parse_args()
