@@ -12,14 +12,15 @@ import pendulum
 import python_graphql_client
 import pandas as pd
 import requests
+import base64
 from time import sleep
 from datetime import datetime
-from typing import Any, Optional, Dict, List, Union
+from typing import Any, Optional, Dict, List, Union, Tuple
 from tqdm.auto import tqdm
 from requests.auth import AuthBase
 from sumatra_client.auth import SDKKeyAuth, CognitoJwtAuth
 from sumatra_client.config import CONFIG
-
+from sumatra_client.workspace import WorkspaceClient
 
 logger = logging.getLogger("sumatra.client")
 
@@ -111,6 +112,7 @@ class Client:
         self,
         instance: Optional[str] = None,
         branch: Optional[str] = None,
+        workspace: Optional[str] = None,
     ):
         """
         Create connection object.
@@ -118,22 +120,76 @@ class Client:
         Arguments:
             instance: Sumatra instance url, e.g. `yourco.sumatra.ai`. If unspecified, the your config default will be used.
             branch: Set default branch. If unspecified, your config default will be used.
+            workspace: Sumatra workspace name to connect to.
         """
         if instance:
             CONFIG.instance = instance
+        self._branch = branch or CONFIG.default_branch
+        self._workspace_arg = workspace or CONFIG.workspace
+        self._workspace = None
+        self._workspace_id = None
         if CONFIG.sdk_key:
+            logger.info("Connecting via SDK key")
             auth: AuthBase = SDKKeyAuth()
             endpoint = CONFIG.sdk_graphql_url
         else:
-            auth = CognitoJwtAuth()
+            auth = CognitoJwtAuth(self.workspace)
             endpoint = CONFIG.console_graphql_url
-
-        self._branch = branch or CONFIG.default_branch
-
         self._gql_client = python_graphql_client.GraphqlClient(
             auth=auth, endpoint=endpoint
         )
 
+    def _choose_workspace(self) -> Tuple[Optional[str], Optional[str]]:
+        if CONFIG.sdk_key:
+            sdk_key_workspace, tenant_id = self._get_workspace_from_sdk_key()
+            if self._workspace_arg and sdk_key_workspace != self._workspace_arg:
+                raise ValueError(
+                    f"SDK Key's workspace: '{sdk_key_workspace}' does not match "
+                    f"chosen workspace: '{self._workspace_arg}'."
+                )
+            return sdk_key_workspace, tenant_id
+        try:
+            workspaces = WorkspaceClient().get_workspaces()
+        except RuntimeError:  # TODO: this is a hack to support v1.2.X
+            tenant_id = WorkspaceClient()._old_tenant()
+            return tenant_id, tenant_id
+        if self._workspace_arg:
+            if self._workspace_arg not in workspaces.index:
+                raise ValueError(
+                    f"Workspace '{self._workspace_arg}' not found. "
+                    f"Choose one of: {workspaces.index.tolist()}."
+                )
+            ws = workspaces.loc[self._workspace_arg]
+            return ws.name, ws.tenant_id
+        if len(workspaces) == 1:
+            ws = workspaces.iloc[0]
+            return ws.name, ws.tenant_id
+        if len(workspaces) > 1:
+            raise RuntimeError(
+                "Unable to determine workspace. "
+                "Specify a workspace or run `sumatra workspace select`."
+            )
+        raise RuntimeError("No workspaces found. Run `sumatra workspace create` first.")
+
+    def _get_workspace_from_sdk_key(self):
+        logger.debug("Fetching workspace from SDK key")
+        query = """
+            query Workspace {
+                workspace {
+                    id
+                    slug
+                }
+            }
+        """
+
+        ret = self._gql_client.execute(query=query)
+
+        if "errors" in ret:
+            raise RuntimeError(ret["errors"][0]["message"])
+
+        d = ret["data"]["workspace"]
+        return d["slug"], d["id"]
+
     @property
     def instance(self) -> str:
         """
@@ -141,6 +197,71 @@ class Client:
         """
         return CONFIG.instance
 
+    @property
+    def workspace(self) -> Optional[str]:
+        """
+        User's current workspace slug, e.g. `my-workspace`
+        """
+        if not self._workspace:
+            self._workspace, self._workspace_id = self._choose_workspace()
+        return self._workspace
+
+    def _tenant_id(self) -> Optional[str]:
+        if not self._workspace_id:
+            self._workspace, self._workspace_id = self._choose_workspace()
+        return self._workspace_id
+
+    def update_workspace(
+        self,
+        name: Optional[str] = None,
+        slug: Optional[str] = None,
+        icon: Optional[bytes] = None,
+    ) -> Dict:
+        """
+        Update workspace metadata.
+
+        Arguments:
+            name: A human readable name for the new workspace
+            slug: Desired slug of the new workspace. Must consist only of letters, numbers, '-', and '_'. If this slug is taken, a random one will be generated instead, which may be changed later.
+            icon: Binary encoding of a PNG image to use as the workspace icon. Max size 50kb
+
+        Returns:
+            A dict of the updated workspace metadata
+        """
+
+        query = """
+            mutation UpdateWorkspace($id: String!, $name: String, $slug: String, $icon: String) {
+                updateTenant(id: $id, name: $name, slug: $slug, icon: $icon) {
+                    id
+                    slug
+                    name
+                }
+            }
+        """
+
+        if icon:
+            icon = base64.b64encode(icon).decode()
+
+        ret = self._gql_client.execute(
+            query=query,
+            variables={
+                "id": self._tenant_id(),
+                "name": name,
+                "slug": slug,
+                "icon": icon,
+            },
+        )
+
+        if "errors" in ret:
+            raise RuntimeError(ret["errors"][0]["message"])
+
+        d = ret["data"]["updateTenant"]
+        return {
+            "id": d["id"],
+            "slug": d["slug"],
+            "name": d["name"],
+        }
+
     @property
     def branch(self) -> str:
         """
@@ -152,18 +273,60 @@ class Client:
     def branch(self, branch: str) -> None:
         self._branch = branch
 
+    def user_email(self) -> str:
+        """
+        Return the email address of the connected user.
+
+        Returns:
+            Email address
+        """
+        logger.debug("Fetching user")
+        query = """
+        query CurrentUser {
+            currentUser {
+                email
+            }
+        }
+        """
+
+        ret = self._gql_client.execute(query=query)
+
+        if "errors" in ret:
+            raise RuntimeError(ret["errors"][0]["message"])
+
+        return ret["data"]["currentUser"]["email"]
+
     def tenant(self) -> str:
         """
         Return the assigned tenant name for the connected user.
+        (DEPRECATED) use .workspace instead
 
         Returns:
-            Tenant name
+            Tenant name (slug)
         """
-        logger.debug("Fetching tenant")
+        if self.version().startswith("v1.2"):
+            return WorkspaceClient()._old_tenant()
+        logger.warn(
+            "tenant() is deprecated and will be removed in the next release. Use .workspace instead."
+        )
+        return self.workspace
+
+    def sdk_key(self) -> str:
+        """
+        Return the SDK key for the connected workspace
+
+        Returns:
+            SDK key
+        """
+        logger.debug("Fetching tenant sdk key")
         query = """
-            query Tenant {
-                tenant {
-                    key
+            query SDKKey {
+                currentTenant {
+                    accessKeys(type: "sdk") {
+                        nodes {
+                            key
+                        }
+                    }
                 }
             }
         """
@@ -173,7 +336,34 @@ class Client:
         if "errors" in ret:
             raise RuntimeError(ret["errors"][0]["message"])
 
-        return ret["data"]["tenant"]["key"]
+        return ret["data"]["currentTenant"]["accessKeys"]["nodes"][0]["key"]
+
+    def api_key(self) -> str:
+        """
+        Return the API key for the connected workspace
+
+        Returns:
+            API key
+        """
+        logger.debug("Fetching tenant api key")
+        query = """
+            query APIKey {
+                currentTenant {
+                    accessKeys(type: "api") {
+                        nodes {
+                            key
+                        }
+                    }
+                }
+            }
+        """
+
+        ret = self._gql_client.execute(query=query)
+
+        if "errors" in ret:
+            raise RuntimeError(ret["errors"][0]["message"])
+
+        return ret["data"]["currentTenant"]["accessKeys"]["nodes"][0]["key"]
 
     def query_athena(self, sql: str) -> pd.DataFrame:
         """
@@ -187,7 +377,7 @@ class Client:
             Result of query as a Pandas dataframe
         """
         session = self._get_session()
-        tenant = self.tenant()
+        tenant = self._tenant_id()
         return wr.athena.read_sql_query(
             boto3_session=session,
             sql=sql,
@@ -950,7 +1140,8 @@ class Client:
             #    selectors.append(time_selector.format(f, f, f))
             else:
                 selectors.append(selector.format(f, f))
-                json_fields.append(f)
+                if f in features:
+                    json_fields.append(f)
 
         where = f"where {where}" if where else ""
         inputs = ""
@@ -958,6 +1149,7 @@ class Client:
             inputs = ', event as "_inputs"'
             json_fields.append("_inputs")
 
+        features = [f'"{feature}"' for feature in features]
         returned = ", ".join(["_id", "_time"] + features)
 
         query = f"""with tmp as (
@@ -1510,6 +1702,7 @@ class Client:
         branch: Optional[str] = None,
     ) -> Materialization:
         """
+        (BETA)
         Recompute historical feature values from LIVE event log on given topology branch.
 
         This is the primary function of the SDK.
@@ -1888,7 +2081,7 @@ class Client:
 
     def get_deps(self, live: bool = False) -> str:
         """
-        Fetch latest dependencies from sever as Scowl source `require` statements.
+        Fetch latest dependencies from server as Scowl source `require` statements.
 
         Arguments:
             live: Return the LIVE versions of dependencies instead of latest.
@@ -2378,198 +2571,6 @@ class Client:
         if "errors" in ret:
             raise RuntimeError(ret["errors"][0]["message"])
 
-    def list_users(self) -> None:
-        """
-        List users.
-        """
-        query = """
-            query ListUsers {
-                users {
-                    nodes {
-                        username
-                    }
-                }
-            }
-        """
-
-        ret = self._gql_client.execute(query=query)
-
-        if "errors" in ret:
-            raise RuntimeError(ret["errors"][0]["message"])
-
-        return [user["username"] for user in ret["data"]["users"]["nodes"]]
-
-    def get_user(self, username: str) -> Dict[str, str]:
-        """
-        Get User.
-        """
-        query = """
-            query User($username: String!) {
-                user(username: $username) {
-                    username
-                    email
-                    role
-                    tenant
-                    admin
-                    status
-                }
-            }
-        """
-
-        ret = self._gql_client.execute(
-            query=query,
-            variables={
-                "username": username,
-            },
-        )
-
-        if "errors" in ret:
-            raise RuntimeError(ret["errors"][0]["message"])
-
-        return ret["data"]["user"]
-
-    def list_tenants(self) -> None:
-        """
-        List tenants.
-        """
-        query = """
-                query ListTenants {
-                    tenants {
-                        nodes {
-                            key
-                        }
-                    }
-                }
-            """
-
-        ret = self._gql_client.execute(query=query)
-
-        if "errors" in ret:
-            raise RuntimeError(ret["errors"][0]["message"])
-
-        return [tenant["key"] for tenant in ret["data"]["tenants"]["nodes"]]
-
-    def set_tenant(self, username: Str, tenant: Str):
-        """
-        Set tenant for user
-        """
-        query = """
-            mutation SetTenant($username: String!, $tenant: String!) {
-                 setUserTenant(username: $username, tenant: $tenant) {
-                    tenant
-                    username
-                  }
-            }
-        """
-
-        ret = self._gql_client.execute(
-            query=query,
-            variables={
-                "username": username,
-                "tenant": tenant,
-            },
-        )
-
-        if "errors" in ret:
-            raise RuntimeError(ret["errors"][0]["message"])
-
-    def set_role(self, username: Str, role: Str):
-        """
-        Set role for user
-        """
-        query = """
-            mutation SetRole($username: String!, $role: String!) {
-                 setUserRole(username: $username, role: $role) {
-                    role
-                    username
-                  }
-            }
-        """
-
-        ret = self._gql_client.execute(
-            query=query,
-            variables={
-                "username": username,
-                "role": role,
-            },
-        )
-
-        if "errors" in ret:
-            raise RuntimeError(ret["errors"][0]["message"])
-
-    def get_keys(self) -> pd.DataFrame:
-        """
-        Get API Keys for tenants
-        """
-        query = """
-            query ApiKeys {
-                tenants {
-                    nodes {
-                        key
-                        apiKey
-                        sdkKey
-                    }
-                }
-            }
-        """
-
-        ret = self._gql_client.execute(query=query)
-        if "errors" in ret:
-            raise RuntimeError(ret["errors"][0]["message"])
-
-        rows = []
-        for tenant in ret["data"]["tenants"]["nodes"]:
-            rows.append((tenant["key"], "api", tenant["apiKey"]))
-            rows.append((tenant["key"], "sdk", tenant["sdkKey"]))
-
-        return (
-            pd.DataFrame(rows, columns=("Tenant", "Type", "Key"))
-            .set_index(["Tenant", "Type"])
-            .sort_index()
-        )
-
-    def get_key_usage(self, start_date=None, end_date=None) -> pd.DataFrame:
-        """
-        Get API Key usage for tenants
-        """
-        query = """
-            query ApiKeyUsage($startDate: DateTime!, $endDate: DateTime!) {
-                apiKeyUsage(startDate: $startDate, endDate: $endDate) {
-                    tenant
-                    apiType
-                    dailyUsage
-                }
-            }
-        """
-
-        end_date = end_date or pendulum.today()
-        start_date = start_date or end_date.subtract(days=6)
-
-        ret = self._gql_client.execute(
-            query=query,
-            variables={
-                "startDate": start_date.to_iso8601_string(),
-                "endDate": end_date.to_iso8601_string(),
-            },
-        )
-
-        if "errors" in ret:
-            raise RuntimeError(ret["errors"][0]["message"])
-
-        days = [d.to_date_string() for d in end_date - start_date]
-        rows = []
-        for usage in ret["data"]["apiKeyUsage"]:
-            dailyUsage = usage["dailyUsage"]
-            if len(dailyUsage) == 0:
-                dailyUsage = [0 for _ in days]
-            rows.append([usage["tenant"], usage["apiType"]] + dailyUsage)
-
-        return (
-            pd.DataFrame(rows, columns=["Tenant", "Type"] + days)
-            .set_index(["Tenant", "Type"])
-            .sort_index()
-        )
-
 
 class ModelVersion:
     """
@@ -2834,7 +2835,7 @@ class TableVersion:
 
 class Materialization:
     """
-    A handle to a server-side materialization job, which enriches a Timeline
+    A handle to a server-side materialization (replay) job, which enriches a Timeline
     (or set of Timelines) by running it (them) through a given Topology.
 
     Objects are not constructed directly. Materializations are returned by methods
@@ -2975,6 +2976,7 @@ class Materialization:
         """
         Return enriched events of specified type. Waits if
         job is still processing.
+        (DEPRECATED) use get_features() instead.
 
         Arguments:
             event_type: Name of event type to fetch.
@@ -2983,6 +2985,23 @@ class Materialization:
         Returns:
             Enriched events
         """
+        logger.warn(
+            "get_events() is deprecated and will be removed in the next release. Use get_features() instead."
+        )
+        return self.get_features(event_type, features)
+
+    def get_features(self, event_type: str, features: List[str] = []) -> pd.DataFrame:
+        """
+        Return feature values for given event type. Waits if
+        job is still processing.
+
+        Arguments:
+            event_type: Name of event type to fetch.
+            features: Feature names to fetch. By default, fetch all.
+
+        Returns:
+            Feature dataframe
+        """
         # if event_type not in self.event_types:
         #    raise RuntimeError(f"Event type '{event_type}' not found.")
 
